#include "common.h"
#include "Archiver7ZIP.h"
#include "../util.h"
#include "../ConfigCode/config.h"
#include "../resource.h"
#include <atlddx.h>		// DDX/DDVgp邽

const struct tagZIPVOLUMEUNIT{
	LPCTSTR DispName;
	LPCTSTR ParamName;
}ZIP_VOLUME_UNIT[]={
	{_T("MB"),_T("m")},
	{_T("KB"),_T("k")},
	{_T("GB"),_T("g")},
	{_T("Byte"),_T("b")}
};
const int ZIP_VOLUME_UNIT_MAX_NUM=sizeof(ZIP_VOLUME_UNIT)/sizeof(tagZIPVOLUMEUNIT);

class C7Zip32VolumeSizeDialog : public CDialogImpl<C7Zip32VolumeSizeDialog>,public CWinDataExchange<C7Zip32VolumeSizeDialog>
{
protected:
public:
	int VolumeSize;
	int SelectIndex;
	CComboBox Combo_VolumeUnit;
	enum {IDD = IDD_DIALOG_7Z_VOLUME_SIZE};

	// DDX}bv
	BEGIN_DDX_MAP(C7Zip32VolumeSizeDialog)
		DDX_INT(IDC_EDIT_7Z_VOLUME_SIZE, VolumeSize)
	END_DDX_MAP()

	// bZ[W}bv
	BEGIN_MSG_MAP_EX(C7Zip32VolumeSizeDialog)
		MSG_WM_INITDIALOG(OnInitDialog)
		COMMAND_ID_HANDLER_EX(IDOK, OnOK)
		COMMAND_ID_HANDLER_EX(IDCANCEL, OnCancel)
	END_MSG_MAP()

	LRESULT OnInitDialog(HWND hWnd, LPARAM lParam){
		Combo_VolumeUnit=GetDlgItem(IDC_COMBO_7Z_VOLUME_UNIT);
		SelectIndex=0;
		for(int i=0;i<ZIP_VOLUME_UNIT_MAX_NUM;i++){
			Combo_VolumeUnit.InsertString(-1,ZIP_VOLUME_UNIT[i].DispName);
		}
		Combo_VolumeUnit.SetCurSel(0);
		return TRUE;
	}

	void OnOK(UINT uNotifyCode, int nID, HWND hWndCtl){
		if(!DoDataExchange(TRUE)){
			return;
		}
		if(VolumeSize<=0){
			::SetFocus(GetDlgItem(IDC_EDIT_7Z_VOLUME_SIZE));
			::SendMessage(GetDlgItem(IDC_EDIT_7Z_VOLUME_SIZE),EM_SETSEL,0,-1);
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_VALUE_MUST_BE_PLUS)));
			return;
		}
		SelectIndex=Combo_VolumeUnit.GetCurSel();
		if(SelectIndex<0||SelectIndex>=ZIP_VOLUME_UNIT_MAX_NUM)return;
		EndDialog(nID);
	}

	void OnCancel(UINT uNotifyCode, int nID, HWND hWndCtl){
		EndDialog(nID);
	}
};

//---------------

CArchiver7ZIP::CArchiver7ZIP()
{
//	TRACE("CArchiver7ZIP()\n");
//	Separator=_T('\\');
	RequiredVersion=431;
	RequiredSubVersion=3;
	DllName=_T("7-ZIP32.DLL");
	Prefix=_T("SevenZip");
}

CArchiver7ZIP::~CArchiver7ZIP()
{
//	TRACE("~CArchiver7ZIP()\n");
	FreeDLL();
}

bool CArchiver7ZIP::Compress(LPCTSTR ArcFileName,std::list<CString> &ParamList,CConfigManager &Config,const PARAMETER_TYPE Type,int Options)
{
	if(!IsOK()){
		return false;
	}

	ASSERT(0!=_tcslen(ArcFileName));
	TRACE("ArcFileName=%s\n",ArcFileName);

	//ɂ̏ꍇ͎ȉ𓀂͏oȂ
	if((Options&COMPRESS_SPLIT)&&(Options&COMPRESS_SFX)){
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_CANNOT_SPLIT_SFX)));
		Options&=~COMPRESS_SFX;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("zip")))return false;
	ASSERT(0!=_tcslen(ResponceFileName));

	//============================================
	// ȉ𓀃t@Cpe|t@C擾
	//============================================
	TCHAR SFXTemporaryFileName[_MAX_PATH+1];
	FILL_ZERO(SFXTemporaryFileName);
	bool bZIPSFX=((0!=(Options&COMPRESS_SFX))&&(PARAMETER_ZIP==Type));
	if(bZIPSFX){
		//2iK쐬
		if(!UtilGetTemporaryFileName(SFXTemporaryFileName,_T("sfx")))return false;
		ASSERT(0!=_tcslen(SFXTemporaryFileName));
		DeleteFile(SFXTemporaryFileName);//S~t@C
	}

	//ȉ𓀗pW[pX
	TCHAR SFXModulePath[_MAX_PATH+1];

	//====================================================
	// X|Xt@CɈkΏۃt@CL
	// A[JCut@C̓R}hCŒڎw肷
	//====================================================
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile)
		{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			return false;
		}

		TRACE("X|Xt@Cւ̏\n");
		std::list<CString>::iterator ite;
		for(ite=ParamList.begin();ite!=ParamList.end();ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE("DLLɓnIvV̐ݒ\n");

	CString Param;//R}hC p[^ obt@

	switch(Type){
	case PARAMETER_ZIP:	//ZIP`ňk
		Config.LoadConfig(CONFIG_ZIP);
		Param+=_T(
			"a "			//k
			"-tzip "		//ZIP`ňk
			"-r0 "			//TufBNg
		);
		switch(Config.ZIP.CompressType){	//k
		case ZIP_COMPRESS_DEFLATE:
			Param+=_T("-mm=Deflate ");
			break;
		case ZIP_COMPRESS_DEFLATE64:
			Param+=_T("-mm=Deflate64 ");
			break;
		case ZIP_COMPRESS_BZIP2:
			Param+=_T("-mm=BZip2 ");
			break;
		case ZIP_COMPRESS_COPY:
			Param+=_T("-mm=Copy ");
			break;
		}
		switch(Config.ZIP.CompressLevel){	//kx
		case ZIP_COMPRESS_LEVEL0:
			Param+=_T("-mx=0 ");
			break;
		case ZIP_COMPRESS_LEVEL5:
			Param+=_T("-mx=5 ");
			break;
		case ZIP_COMPRESS_LEVEL9:
			Param+=_T("-mx=9 ");
			break;
		}
		if((ZIP_COMPRESS_DEFLATE==Config.ZIP.CompressType)||(ZIP_COMPRESS_DEFLATE64==Config.ZIP.CompressType)){
			if(Config.ZIP.SpecifyDeflateMemorySize){
				CString temp;
				temp.Format(_T("-mfb=%d "),Config.ZIP.DeflateMemorySize);
				Param+=temp;
			}
			if(Config.ZIP.SpecifyDeflatePassNumber){
				CString temp;
				temp.Format(_T("-mpass=%d "),Config.ZIP.DeflatePassNumber);
				Param+=temp;
			}
		}
		if(Options&COMPRESS_PASSWORD){	//pX[ht
			Param+=_T("-p ");
		}
		if(bZIPSFX){
			//BZip2̎ȉ𓀂̓T|[gĂȂ
			if(ZIP_COMPRESS_BZIP2==Config.ZIP.CompressType){
				ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_ZIP_SFX_DONT_SUPPORT_BZIP2)));
				DeleteFile(ResponceFileName);
				return false;
			}
			FILL_ZERO(SFXModulePath);
			LPTSTR lptemp;
			if(!SearchPath(NULL,_T("SFX32GUI.DAT"),NULL,_MAX_PATH,SFXModulePath,&lptemp)){
//			if(!UtilGetSFXModuleFullPath(SFXModulePath,_T("SFX32GUI.DAT"))){
				CString msg;
				msg.Format(IDS_ERROR_SFX_MODULE_NOT_FOUND,_T("SFX32GUI.DAT"));
				DeleteFile(ResponceFileName);
				ErrorMessage(msg);
				return false;
			}
		}
		break;
	case PARAMETER_7Z:	//7z`ňk
		Config.LoadConfig(CONFIG_7Z);
		Param+=_T(
			"a "			//k
			"-t7z "			//7-Zip`ňk
			"-r0 "			//TufBNg
		);
		if(Config.SEVEN_ZIP.UsePreset){
			//vZbgk[h
			switch(Config.SEVEN_ZIP.CompressLevel){	//kx
			case SEVEN_ZIP_COMPRESS_LEVEL0:
				Param+=_T("-mx=0 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL1:
				Param+=_T("-mx=1 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL5:
				Param+=_T("-mx=5 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL7:
				Param+=_T("-mx=7 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL9:
				Param+=_T("-mx=9 ");
				break;
			}
			if(Config.SEVEN_ZIP.LZMA_MultiThread){
				Param+=_T("-mmt=on ");	//}`Xbh(LZMÂ)
			}
		}else{
			switch(Config.SEVEN_ZIP.CompressType){	//k
			case SEVEN_ZIP_COMPRESS_LZMA:
				if(Config.SEVEN_ZIP.LZMA_MultiThread){
					Param+=_T("-mmt=on ");	//}`Xbh(LZMÂ)
				}
				Param+=_T("-m0=LZMA:a=");
				switch(Config.SEVEN_ZIP.LZMA_Mode){	//LZMAk[h
				case SEVEN_ZIP_LZMA_MODE0:
					Param+=_T("0 ");
					break;
				case SEVEN_ZIP_LZMA_MODE1:
					Param+=_T("1 ");
					break;
				case SEVEN_ZIP_LZMA_MODE2:
					Param+=_T("2 ");
					break;
				}
				break;
			case SEVEN_ZIP_COMPRESS_PPMD:
				Param+=_T("-m0=PPMd ");
				break;
			case SEVEN_ZIP_COMPRESS_BZIP2:
				Param+=_T("-m0=BZip2 ");
				break;
			case SEVEN_ZIP_COMPRESS_DEFLATE:
				Param+=_T("-m0=Deflate ");
				break;
			case SEVEN_ZIP_COMPRESS_COPY:
				Param+=_T("-m0=Copy ");
				break;
			}
		}
		if(Config.SEVEN_ZIP.SolidMode){	//\bh[h
			Param+=_T("-ms=on ");
		}else{
			Param+=_T("-ms=off ");
		}
		//------------
		// wb_k
		//------------
		if(Config.SEVEN_ZIP.HeaderCompression){	//wb_k
			Param+=_T("-mhc=on ");
			if(Config.SEVEN_ZIP.FullHeaderCompression){	//wb_Sk
				Param+=_T("-mhcf=on ");
			}else{
				Param+=_T("-mhcf=off ");
			}
		}else{
			Param+=_T("-mhc=off ");
			Param+=_T("-mhcf=off ");
		}

		if(Options&COMPRESS_PASSWORD){	//pX[ht
			Param+=_T("-p ");
			if(Config.SEVEN_ZIP.HeaderEncryption){	//wb_Í
				Param+=_T("-mhe=on ");
			}
			else{
				Param+=_T("-mhe=off ");
			}
		}
		if(Options&COMPRESS_SFX){
			//BZip2̎ȉ𓀂̓T|[gĂȂ
			if(SEVEN_ZIP_COMPRESS_BZIP2==Config.SEVEN_ZIP.CompressType){
				ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_7Z_SFX_DONT_SUPPORT_BZIP2)));
				DeleteFile(ResponceFileName);
				return false;
			}
			Param+=_T("-sfx ");
		}
		break;
	}
	//
	if(Options&COMPRESS_SPLIT){
		C7Zip32VolumeSizeDialog vsd;
		if(IDOK!=vsd.DoModal())return false;
		CString temp;
		temp.Format(_T("-v%d%s "),vsd.VolumeSize,ZIP_VOLUME_UNIT[vsd.SelectIndex].ParamName);
		Param+=temp;
	}


	//kt@Cw
	if(bZIPSFX){
		Param+=_T("\"");
		Param+=SFXTemporaryFileName;
		Param+=_T("\" ");
	}
	else{
		Param+=_T("\"");
		Param+=ArcFileName;
		Param+=_T("\" ");
	}

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE("ArchiveHandler Commandline Parameter:%s\n",Param);

	TRACE("ArchiveHandlerĂяo\n");
	TCHAR Log[LOG_BUFFER_SIZE];
	FILL_ZERO(Log);
	int Ret=ArchiveHandler(NULL,Param,Log,LOG_BUFFER_SIZE-1);

	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	//G[Oo
	if(!bZIPSFX||0!=Ret){
		if(bZIPSFX){
			DeleteFile(SFXTemporaryFileName);
		}
		switch(Config.Common.LogViewEvent){
		case LOGVIEW_ON_ERROR:
			if(Ret){
				CLogDialog LogDlg;
				LogDlg.SetData(Log);
				LogDlg.DoModal();
			}
			break;
		case LOGVIEW_ALWAYS:
			CLogDialog LogDlg;
			LogDlg.SetData(Log);
			LogDlg.DoModal();
			break;
		}

		return 0==Ret;
	}

	//==================================
	// ȉ𓀏ɂɕϊ(oCi)
	//==================================
	//o͐t@CJ
	HANDLE hArcFile=CreateFile(ArcFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(INVALID_HANDLE_VALUE==hArcFile)
	{
		CString msg;
		msg.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
		ErrorMessage(msg);
		DeleteFile(ArcFileName);
		DeleteFile(SFXTemporaryFileName);
		return false;
	}
	{
		//SFXW[ǂݎ胂[hŊJ
		HANDLE hSFXModuleFile=CreateFile(SFXModulePath,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hSFXModuleFile)
		{
			CString msg;
			msg.Format(IDS_ERROR_SFX_MODULE_CANNOT_ACCESS,SFXModulePath);
			ErrorMessage(msg);
			CloseHandle(hArcFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		//SFXW[̒gRs[
		int Result=UtilAppendFile(hArcFile,hSFXModuleFile);
		if(Result>0){	//ǂݎG[
			CString msg;
			msg.Format(IDS_ERROR_SFX_MODULE_CANNOT_ACCESS,SFXModulePath);
			ErrorMessage(msg);
			CloseHandle(hArcFile);
			CloseHandle(hSFXModuleFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		else if(Result<0){	//݃G[
			CString msg;
			msg.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
			ErrorMessage(msg);
			CloseHandle(hArcFile);
			CloseHandle(hSFXModuleFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		CloseHandle(hSFXModuleFile);
	}
	{
		//e|t@Cǂݎ胂[hŊJ
		HANDLE hSFXTempFile=CreateFile(SFXTemporaryFileName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hSFXTempFile)
		{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			CloseHandle(hArcFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		//e|t@C̒gRs[
		int Result=UtilAppendFile(hArcFile,hSFXTempFile);
		if(Result>0){	//ǂݎG[
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			CloseHandle(hArcFile);
			CloseHandle(hSFXTempFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		else if(Result<0){	//݃G[
			CString msg;
			msg.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
			ErrorMessage(msg);
			CloseHandle(hArcFile);
			CloseHandle(hSFXTempFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		CloseHandle(hSFXTempFile);
		DeleteFile(SFXTemporaryFileName);
	}
	CloseHandle(hArcFile);

	return true;
}

//bSafeArchive͖
bool CArchiver7ZIP::Extract(LPCTSTR ArcFileName,CConfigManager &Config,bool,LPCTSTR OutputDir)
{
	if(!IsOK()){
		return false;
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE("DLLɓnIvV̐ݒ\n");

	CString Param;//R}hC p[^ obt@

	//𓀃p[^
	Param+=_T(
		"x "			//
	);
	if(Config.Common.Extract.ForceOverwrite){
		//㏑
		Param+=_T("-aoa ");
	}

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//o͐w
	Param+=_T("-o\"");
	Param+=OutputDir;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE("ArchiveHandler Commandline Parameter:%s\n",Param);

	TRACE("ArchiveHandlerĂяo\n");
	CString Log;
	int Ret=ArchiveHandler(NULL,Param,Log.GetBuffer(LOG_BUFFER_SIZE),LOG_BUFFER_SIZE-1);
	Log.ReleaseBuffer();

	//G[Oo
	switch(Config.Common.LogViewEvent){
	case LOGVIEW_ON_ERROR:
		if(Ret){
			CLogDialog LogDlg;
			LogDlg.SetData(Log);
			LogDlg.DoModal();
		}
		break;
	case LOGVIEW_ALWAYS:
		CLogDialog LogDlg;
		LogDlg.SetData(Log);
		LogDlg.DoModal();
		break;
	}

	return 0==Ret;
}

//X|Xt@CɃt@CGXP[vsŏށB
//Lȃt@CnhNULLłȂt@CnƁB
void CArchiver7ZIP::WriteResponceFile(HANDLE hFile,LPCTSTR fname)
{
	TCHAR Buffer[_MAX_PATH+1];
	FILL_ZERO(Buffer);

	_tcsncpy(Buffer,fname,_MAX_PATH);
	if(PathIsDirectory(Buffer)){
		PathUnquoteSpaces(Buffer);
		PathAppend(Buffer,_T("*"));
	}
	PathQuoteSpaces(Buffer);

	DWORD dwWritten=0;
	//t@Co
	WriteFile(hFile,Buffer,_tcslen(Buffer)*sizeof(TCHAR),&dwWritten,NULL);

	//t@C؂邽߂̉s
	TCHAR CRLF[]=_T("\r\n");
	WriteFile(hFile,&CRLF,_tcslen(CRLF)*sizeof(TCHAR),&dwWritten,NULL);
}


//=============================================================
// SevenZipGetFileName()̏o͌ʂɁAi[ꂽt@C
// pXĂ邩ǂʂAdtH_쐬h
//=============================================================
bool CArchiver7ZIP::ExamineArchive(LPCTSTR ArcFileName,bool bSkipDir,bool &bInFolder,bool &,CString &BaseDir)
{
	if(bSkipDir){
		TRACE("dtH_\n");
		return true;
	}
	return _ExamineArchiveFast(ArcFileName,bInFolder,BaseDir);
}

bool CArchiver7ZIP::ExtractSpecifiedOnly(LPCTSTR ArcFileName,LPCTSTR OutputDir,std::list<CString> &FileList)
{
	if(!IsOK()){
		return false;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("7zp")))return false;
	ASSERT(0!=_tcslen(ResponceFileName));

	//𓀑Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile)
		{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			return false;
		}

		std::list<CString>::iterator ite=FileList.begin();
		const std::list<CString>::iterator end=FileList.end();
		for(;ite!=end;ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//𓀃p[^
	Param+=_T(
		"e "		//fBNgȂŉ
		"-r- "		//ċA
	);

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//o͐w
	Param+=_T("-o\"");
	Param+=OutputDir;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	TRACE("ArchiveHandlerĂяo\nCommandline Parameter:%s\n",Param);
	CString Log;
	int Ret=ArchiveHandler(NULL,Param,Log.GetBuffer(LOG_BUFFER_SIZE),LOG_BUFFER_SIZE-1);
	Log.ReleaseBuffer();
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	if(Ret){
		CLogDialog LogDlg;
		LogDlg.SetData(Log);
		LogDlg.DoModal();
	}
	return 0==Ret;
}


bool CArchiver7ZIP::DeleteFileFromArchive(LPCTSTR ArcFileName,std::list<CString>& FileList)
{
	if(!IsOK()){
		return false;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("7zp")))return false;
	ASSERT(0!=_tcslen(ResponceFileName));

	//폜Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile)
		{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS)));
			return false;
		}

		std::list<CString>::iterator ite=FileList.begin();
		const std::list<CString>::iterator end=FileList.end();
		for(;ite!=end;ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//폜p[^
	Param+=_T(
		"d "	//폜
		"-r- "	//ċA
	);

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");


	TRACE("ArchiveHandlerĂяo\nCommandline Parameter:%s\n",Param);
	CString Log;
	int Ret=ArchiveHandler(NULL,Param,Log.GetBuffer(LOG_BUFFER_SIZE),LOG_BUFFER_SIZE-1);
	Log.ReleaseBuffer();
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	if(Ret){
		CLogDialog LogDlg;
		LogDlg.SetData(Log);
		LogDlg.DoModal();
	}
	return 0==Ret;
}

bool CArchiver7ZIP::TestArchive(LPCSTR ArcFileName)
{
	TRACE("TestArchive() called.\n");
	ASSERT(IsOK());
	if(!IsOK()){
		return false;
	}

	//tR}hɂeXgĂ
	CString Param=_T(
		"t "			//eXg

		"\""
	);
	Param+=ArcFileName;
	Param+=_T("\"");

	CString Log;
	ArchiveHandler(NULL,Param,Log.GetBuffer(LOG_BUFFER_SIZE),LOG_BUFFER_SIZE-1);
	Log.ReleaseBuffer();

	//O\
	CLogDialog LogDlg;
	LogDlg.SetData(Log);
	LogDlg.DoModal();
	return true;
}
