#include "common.h"
#include "util.h"
#include "resource.h"

#include <mbctype.h>
#include "ConfigCode/config.h"

//G[bZ[W\
int ErrorMessage(LPCTSTR msg)
{
	TRACE("ErrorMessage:"),TRACE(msg),TRACE("\n");
	return MessageBox(NULL,msg,UtilGetMessageCaption(),MB_OK|MB_ICONSTOP);
}

//bZ[WLvV擾
LPCTSTR UtilGetMessageCaption()
{
	const static CString strCaption(MAKEINTRESOURCE(IDS_MESSAGE_CAPTION));
	return strCaption;
}

bool UtilIsSBSymbolorNumber(LPCTSTR String,int Index)
{
	static const TCHAR Table[]=_T("0123456789./*-+{}[]@`:;!\"#$%&\'()_><=~^|,\\");
	static const int Length=_tcslen(Table);

	if(_MBC_SINGLE!=_mbsbtype((const unsigned char*)String,Index))return false;
	const TCHAR c=String[Index];
	for(int i=0;i<Length;i++){
		if(Table[i]==c)return true;
	}
	return false;
}


bool UtilGetTemporaryFileName(LPTSTR fname,LPCTSTR prefix)
{
	TCHAR TempDirPath[_MAX_PATH+1];
	FILL_ZERO(TempDirPath);
	GetTempPath(_MAX_PATH,TempDirPath);
	if(!GetTempFileName(TempDirPath,prefix,0,fname))
	{
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE)));
		return false;
	}
	return true;
}

bool UtilDeletePath(LPCTSTR PathName)
{
	if(PathIsDirectory(PathName)){//fBNg
		//t@C̑Wɖ߂
		SetFileAttributes(PathName,FILE_ATTRIBUTE_NORMAL);
		if(RemoveDirectory(PathName))return true;
	}
	else if(PathFileExists(PathName)){//t@C
		//t@C̑Wɖ߂
		SetFileAttributes(PathName,FILE_ATTRIBUTE_NORMAL);
		if(DeleteFile(PathName))return true;
	}
	return false;
}

//bDeleteParent=truêƂAPathg폜
//bDeleteParent=falsêƂ́APath̒g폜
bool UtilDeleteDir(LPCTSTR Path,bool bDeleteParent)
{
	WIN32_FIND_DATA lp;

	TCHAR FindParam[_MAX_PATH+1];
	FILL_ZERO(FindParam);
	strncpy(FindParam,Path,_MAX_PATH);
	PathAppend(FindParam,_T("*"));
	HANDLE h=FindFirstFile(FindParam,&lp);

	bool bRet=true;

	do{
		if((lp.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)&& 0!=_tcscmp(lp.cFileName,_T("..")) && 0!=_tcscmp(lp.cFileName,_T(".")))
		{
			CString SubPath=Path;
			SubPath+=_T("\\");
			SubPath+=lp.cFileName;

			bRet=bRet&&UtilDeleteDir(SubPath,true);
		}
		if((lp.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)!=FILE_ATTRIBUTE_DIRECTORY)
		{
			// lp.cFileNameŃt@C
			TCHAR FileName[_MAX_PATH+1];
			FILL_ZERO(FileName);
			strncpy(FileName,Path,_MAX_PATH);
			PathAppend(FileName,lp.cFileName);

			bRet=bRet&&UtilDeletePath(FileName);
		}
	}while(FindNextFile(h,&lp));

	FindClose(h);
	if(bDeleteParent){
		bRet=bRet&&UtilDeletePath(Path);
	}

	return bRet;
}

bool UtilIsWindowsNT()
{
	OSVERSIONINFO osi;
	FILL_ZERO(osi);
	osi.dwOSVersionInfoSize=sizeof(osi);
	GetVersionEx(&osi);
	return (VER_PLATFORM_WIN32_NT==osi.dwPlatformId);
}


//=======================================================
//From http://www.athomejp.com/goldfish/api/shortcut.asp
//=======================================================
HRESULT UtilCreateShortcut(LPCTSTR lpszPathLink,LPCTSTR lpszPathTarget,LPCTSTR lpszArgs,LPCTSTR lpszIconPath,int iIcon,LPCTSTR lpszDescription)
{
	HRESULT hRes;
	IShellLink* psl =NULL;

//	// COMC^[tFCX
//	CoInitialize(nULL);

	// IShellLink C^[tFCX擾
	hRes = CoCreateInstance(
		CLSID_ShellLink,NULL,
		CLSCTX_INPROC_SERVER,
		IID_IShellLink,
		(LPVOID *)&psl);

	if(FAILED(hRes)){
		//s
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_CREATE_SHORTCUT)));
		return hRes;
	}

	IPersistFile* ppf=NULL;
	// LinkIuWFNg̃pXݒ(Ƃ΁A
	// C:\Windows\notepad.exeȂ)
	psl->SetPath(lpszPathTarget);
	//ݒ
	psl->SetArguments(lpszArgs);
	//ACRݒ
	psl->SetIconLocation(lpszIconPath,iIcon);
	//ݒ
	psl->SetDescription(lpszDescription);
	// IPersistFileC^[tFCX擾A
	// LinkpXt@CۑB
	// ́AIShellLink̒
	// fBXN֘AIuWFNg
	// 擾ĂƎvĂB
	hRes =psl->QueryInterface( IID_IPersistFile,(void**)&ppf);
	if(FAILED(hRes)){
		//ppf̎擾Ɏs
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_CREATE_SHORTCUT)));
		psl->Release();
		return hRes;
	}

	WCHAR wsz[MAX_PATH];

	// pX}`oCgɕϊ
	// ́Å֐Save
	// }`oCg̈̂
	MultiByteToWideChar(CP_ACP, 0,lpszPathLink, -1,wsz, _MAX_PATH);

	// fBXNɕۑ
	hRes = ppf->Save(wsz, TRUE);
	ppf->Release();
	psl->Release();
	return hRes;
}



//ǂݎ茳t@Cݐt@Cseek̂ӏɃRs[ď
//߂l̓G[ȂȂ0,ǂݎG[1,݃G[-1
int UtilAppendFile(HANDLE hWriteTo,HANDLE hReadFrom)
{
	//16KBRs[
	DWORD dwRead=0,dwWrite=0;
	std::vector<BYTE> Buffer(16*1024);
	for(;;){
		if(!ReadFile(hReadFrom,&Buffer.at(0),16*1024,&dwRead,NULL)){
			return 1;
		}
		if(0==dwRead){
			break;
		}
		if(!WriteFile(hWriteTo,&Buffer.at(0),dwRead,&dwWrite,NULL)){
			return -1;
		}
		if(dwRead!=dwWrite){
			return -1;
		}
	}
	return 0;
}

void UtilModifyPath(CString &strPath)
{
	// pX̏C
	strPath.Replace(_T("/"),_T("\\"));	//pX؂蕶

	int Ret=0;
	do{
		Ret=strPath.Replace(_T("\\\\"),_T("\\"));	//\\\ɒuĂ
	}while(Ret!=0);
	strPath.Replace(_T("..\\"),_T("__\\"));	//΃pXw..__ɒu
	strPath.Replace(_T(":"),_T("__"));	//hCuu(C:C__)
}

BOOL UtilMoveFileToRecycleBin(LPCTSTR lpFileName)
{
	ASSERT(lpFileName);
	if(!lpFileName)return false;
	TCHAR Buffer[_MAX_PATH+2]={0};
	_tcsncpy(Buffer,lpFileName,_MAX_PATH);

	SHFILEOPSTRUCT shfo={0};
	shfo.wFunc=FO_DELETE;	//폜
	shfo.pFrom=Buffer;//t@C̃Xg\0\0ŏIKvL
	shfo.fFlags=FOF_SILENT/*i󋵂\Ȃ*/|FOF_ALLOWUNDO/*UNDOt;ݔ*/|FOF_NOCONFIRMATION/*mF_CAO\Ȃ*/;
	return SHFileOperation(&shfo);
}


void UtilAskAndCheckUpdate(CConfigManager &Config)
{
	if(difftime(time(NULL),Config.Update.LastTime)<Config.Update.Interval*60*60*24){
		return;
	}
	if(IDYES!=MessageBox(NULL,CString(MAKEINTRESOURCE(IDS_ASK_UPDATE_DLL)),UtilGetMessageCaption(),MB_YESNO|MB_ICONQUESTION)){
		//Abvf[gȂȂXV㏑
		//CaldixINIt@C
		LPCTSTR CaldixIniPath=Config.GetCaldixIniPath();
		CString Buffer;
		Buffer.Format(_T("%ld"),time(NULL));
		WritePrivateProfileString(_T("Update"),_T("LastTime"),Buffer,CaldixIniPath);
		return;
	}
	//------------------------
	// LhaForge{̂̃pX擾
	//------------------------
	TCHAR ExePath[_MAX_PATH+1];
	FILL_ZERO(ExePath);
	GetModuleFileName(GetModuleHandle(NULL), ExePath, _MAX_PATH);

	PathRemoveFileSpec(ExePath);
	PathAppend(ExePath,CString(MAKEINTRESOURCE(IDS_CALDIX_EXE_NAME)));
	if(32>=(int)ShellExecute(NULL,NULL,ExePath,NULL,NULL,SW_RESTORE)){
		//sG[
		LPVOID lpMsgBuf;
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,NULL,GetLastError(),MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),(LPTSTR)&lpMsgBuf, 0, NULL);

		CString msg;
		msg.Format(IDS_ERROR_CANNOT_EXECUTE,ExePath,lpMsgBuf);
		LocalFree(lpMsgBuf);

		ErrorMessage(msg);
	}
}

//From http://techtips.belution.com/ja/vc/0012/
void UtilSetAbsoluteForegroundWindow(HWND hWnd)
{
	int nTargetID, nForegroundID;
	DWORD sp_time;

	// tHAOEhEBhE쐬XbhID擾
	nForegroundID = GetWindowThreadProcessId(GetForegroundWindow(), NULL);
	// ړĨEBhE쐬XbhID擾
	nTargetID = GetWindowThreadProcessId(hWnd, NULL );

	// Xbh̃CvbgԂѕt
	AttachThreadInput(nTargetID, nForegroundID, TRUE );  // TRUE Ōѕt

	// ݂̐ݒ sp_time ɕۑ
	SystemParametersInfo( SPI_GETFOREGROUNDLOCKTIMEOUT,0,&sp_time,0);
	// EBhE̐؂ւԂ 0ms ɂ
	SystemParametersInfo( SPI_SETFOREGROUNDLOCKTIMEOUT,0,(LPVOID)0,0);

	// EBhEtHAOEhɎĂ
	SetForegroundWindow(hWnd);

	// ݒɖ߂
	SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT,0,&sp_time,0);

	// Xbh̃CvbgԂ؂藣
	AttachThreadInput(nTargetID, nForegroundID, FALSE );  // FALSE Ő؂藣
}

//z̒Ɏw肳ꂽLtrueԂ
int UtilCheckNumberArray(const int *lpcArray,size_t size,int c)
{
	for(size_t i=0;i<size;i++){
		if(lpcArray[i]==c)return i;
	}
	return -1;
}

//t@Cw肵2̏[]邩ǂ;ۂD
bool UtilPathAcceptSpec(LPCTSTR lpszPath,const CString& strDeny,const CString &strAccept,bool bDenyOnly)
{
	for(int i=0;i<2;i++){
		if(bDenyOnly&&i==1){
			//ۂ邩ǂ̂݃`FbN
			return true;
		}
		//i=0:ۂ̃`FbN,i=1:̃`FbN
		bool bAccepted=(i==0?false:true);
		const CString& strBuf=(i==0?strDeny:strAccept);
		int Index=0,CopyFrom=0;

		while(true){
			Index=strBuf.Find(_T(';'),Index);
			CString strMatchSpec;
			if(-1==Index){
				strMatchSpec=strBuf.Mid(CopyFrom);
			}
			else{
				strMatchSpec=strBuf.Mid(CopyFrom,Index-CopyFrom);
			}
			if(!strMatchSpec.IsEmpty()){	//gqmF
				strMatchSpec.Insert(0,"*");	//.ext->*.ext
				if(PathMatchSpec(lpszPath,strMatchSpec)){
					return bAccepted;
				}
			}
			if(-1==Index){	//I
				break;
			}
			else{
				Index++;
				CopyFrom=Index;
			}
		}
	}
	//
	return false;
}

//X|Xt@Cǂݎ
bool UtilReadFromResponceFile(LPCSTR lpszRespFile,std::list<CString> &FileList)
{
	ASSERT(lpszRespFile);
	if(!lpszRespFile)return false;
	if(!PathFileExists(lpszRespFile))return false;

	HANDLE hFile=CreateFile(lpszRespFile,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(INVALID_HANDLE_VALUE==hFile)return false;

	//4GBzt@C͈Ȃ
	const DWORD dwSize=GetFileSize(hFile,NULL);
	std::vector<char> cReadBuffer;
	cReadBuffer.resize(dwSize+2);
	DWORD dwRead;
	//---ǂݍ
	if(!ReadFile(hFile,&cReadBuffer[0],dwSize,&dwRead,NULL)||dwSize!=dwRead){
		CloseHandle(hFile);
		return false;
	}
	CloseHandle(hFile);

	//
	cReadBuffer[dwSize]='\0';
	LPCTSTR p=&cReadBuffer[0];
	const LPCSTR end=&cReadBuffer[dwSize]+1;
	CString strLine;
	for(;p!=end;p++){
		if(*p=='\n'||*p=='\r'||*p=='\0'){
			if(!strLine.IsEmpty()){
				TCHAR Buffer[_MAX_PATH+2];
				_tcsncpy(Buffer,strLine,_MAX_PATH);
				PathUnquoteSpaces(Buffer);	//""O
				FileList.push_back(Buffer);
			}
			strLine.Empty();
		}
		else 
			strLine+=*p;
	}
	return true;
}

//INIɐ𕶎Ƃď
BOOL UtilWritePrivateProfileInt(LPCTSTR lpAppName,LPCTSTR lpKeyName,INT nData,LPCTSTR lpFileName)
{
	TCHAR Buffer[32]={0};
	wsprintf(Buffer,_T("%d"),nData);
	return ::WritePrivateProfileString(lpAppName,lpKeyName,Buffer,lpFileName);
}

//tH_t@C(fBNg͏)ċA
bool UtilRecursiveEnumFile(LPCTSTR lpszRoot,std::list<CString> &rFileList)
{
	CFindFile cFindFile;
	TCHAR szPath[_MAX_PATH+1];
	_tcsncpy(szPath,lpszRoot,_MAX_PATH);
	PathAppend(szPath,_T("*"));

	BOOL bContinue=cFindFile.FindFile(szPath);
	while(bContinue){
		if(!cFindFile.IsDots()){
			if(cFindFile.IsDirectory()){
				UtilRecursiveEnumFile(cFindFile.GetFilePath(),rFileList);
			}else{
				rFileList.push_back(cFindFile.GetFilePath());
			}
		}
		bContinue=cFindFile.FindNextFile();
	}

	return !rFileList.empty();
}

//INIɎw肳ꂽZNVȂtrueԂ
bool UtilCheckINISectionExists(LPCTSTR lpAppName,LPCTSTR lpFileName)
{
	TCHAR szBuffer[10];
	DWORD dwRead=GetPrivateProfileSection(lpAppName,szBuffer,9,lpFileName);
	return dwRead>0;
}

class CInputDialog : public CDialogImpl<CInputDialog>,public CWinDataExchange<CInputDialog>
{
protected:
	CString &strInput;
	CString strMessage;
public:
	enum {IDD = IDD_DIALOG_INPUT};
	CInputDialog(LPCTSTR lpszMessage,CString &str):strInput(str),strMessage(lpszMessage){}

	// DDX}bv
	BEGIN_DDX_MAP(CInputDialog)
		DDX_TEXT(IDC_EDIT_INPUT,strInput)
	END_DDX_MAP()

	// bZ[W}bv
	BEGIN_MSG_MAP_EX(CInputDialog)
		MSG_WM_INITDIALOG(OnInitDialog)
		COMMAND_ID_HANDLER_EX(IDOK, OnButton)
		COMMAND_ID_HANDLER_EX(IDCANCEL, OnButton)
	END_MSG_MAP()

	LRESULT OnInitDialog(HWND hWnd, LPARAM lParam){
		//bZ[Wݒ
		CStatic cStatic=GetDlgItem(IDC_STATIC_MESSAGE);
		cStatic.SetWindowText(strMessage);

		//DDXݒ
		DoDataExchange(FALSE);
		return TRUE;
	}
	void OnButton(UINT uNotifyCode, int nID, HWND hWndCtl){
		//DDXݒ
		DoDataExchange(TRUE);
		EndDialog(nID);
	}
};

//͂
bool UtilInputText(LPCTSTR lpszMessage,CString &strInput)
{
	CInputDialog dlg(lpszMessage,strInput);
	return IDOK==dlg.DoModal();
}

